package com.guams.review.model;

import com.guams.review.model.dao.Comment;
import com.guams.review.model.dao.CommentIds;
import org.springframework.data.jdbc.repository.query.Modifying;
import org.springframework.data.jdbc.repository.query.Query;
import org.springframework.data.repository.CrudRepository;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

public interface CommentRepository extends CrudRepository<Comment, Long> {

    List<Comment> findAll();

    @Query("select post_fk, comment_fk, author_fk from comment_association where comment_fk = :commentId")
    Optional<CommentIds> getCommentIdsByCommentId(Long commentId);

    @Query("select distinct comment.id, comment.content, comment.comment_date, comment.is_updated from comment join comment_association on " +
            "comment_association.post_fk = :commentId")
    List<Comment> getCommentsByCommentId(Long commentId);

    @Modifying
    @Query("delete from comment_association where comment_fk = :commentId")
    void deleteAssociationByCommentId(Long commentId);

    @Modifying
    @Query("insert into comment_association(comment_fk, post_fk, author_fk) values (:commentId, :postId, :authorId);")
    void associateCommentToUserAndPost(UUID authorId, Long postId, Long commentId);
}
