drop table if exists comment_association;
drop table if exists comment;
drop table if exists publication;
drop table if exists post;
drop table if exists author;

create table author
(
    id              uuid        default gen_random_uuid() primary key,
    name            varchar(255) unique          not null,
    password        text                         not null,
    profile_picture bytea,
    role            varchar(10) default 'READER' not null
);

create table post
(
    id               serial primary key,
    description      varchar(512) not null,
    illustration     bytea,
    title            varchar(50)  not null,
    body             text         not null,
    category         varchar(50)  not null,
    publication_date timestamp    not null,
    is_updated       boolean default false
);

create table publication
(
    author_fk uuid,
    post_fk   serial,
    foreign key (author_fk) references author (id),
    foreign key (post_fk) references post (id)
);

create table comment
(
    id           serial primary key,
    content      varchar(512) not null,
    comment_date timestamp    not null,
    is_updated   boolean default false
);

create table comment_association
(
    ca_pk      serial primary key,
    comment_fk serial,
    post_fk    serial,
    author_fk  uuid,
    foreign key (comment_fk) references comment (id),
    foreign key (post_fk) references post (id),
    foreign key (author_fk) references author (id)
);